@@ -117,6 +117,7 @@ def self_test_suite():
         'diff_tree',
         'fastexport',
         'file',
+        'grafts',
         'hooks',
         'index',
         'lru_cache',
@@ -125,6 +126,7 @@ def self_test_suite():
         'missing_obj_finder',
         'pack',
         'patch',
+        'porcelain',
         'protocol',
         'repository',
         'server',
